package org.nakedobjects.nos.store.file;

import org.nakedobjects.noa.persist.ObjectPersistenceException;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.persist.SerialOid;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;

import java.io.File;
import java.io.FilenameFilter;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;


public class XmlDataManagerTest extends TestCase {
    protected XmlDataManager manager;
    private TestSystem system;

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(XmlDataManagerTest.class);
    }

    protected void setUp() throws Exception {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);

        system = new TestSystem();
        system.init();

        FileVersion.setClock(new TestClock());

        clearTestDirectory();
        manager = new XmlDataManager(new XmlFile("tmp/tests"));
        
        
        NakedObjectSpecification type = new TestSpecification();
        system.addSpecificationToLoader(type);
        
        system.addSpecificationToLoader(Role.class.getName());
        system.addSpecificationToLoader(Team.class.getName());
    }
    
    protected void tearDown() throws Exception {
        system.shutdown();
    }

    protected static void clearTestDirectory() {
        File directory = new File("tmp" + File.separator + "tests");
        String[] files = directory.list(new FilenameFilter() {
            public boolean accept(final File arg0, final String name) {
                return name.endsWith(".xml");
            }
        });

        if (files != null) {
            for (int f = 0; f < files.length; f++) {
                new File(directory, files[f]).delete();
            }
        }
    }

    public void testCreateOid() throws Exception {
        SerialOid oid = manager.createOid();
        long start = oid.getSerialNo();
        long next = start + 1;
        for (int i = 0; i < 3; i++) {
            oid = manager.createOid();
            assertEquals(next++, oid.getSerialNo());
        }
    }

    public void testWriteReadTypeOidAndVersion() {
        ObjectData data = createData(Role.class, 99, new FileVersion("user", 19));
        manager.insertObject(data);

        ObjectData read = manager.loadObjectData(data.getOid());

        assertEquals(data.getOid(), read.getOid());
        assertEquals(data.getClassName(), read.getClassName());
        assertEquals(data.getVersion(), read.getVersion());
    }

    public void testNextId() throws Exception {
        long first = manager.nextId();
        assertEquals(first + 1, manager.nextId());
        assertEquals(first + 2, manager.nextId());
        assertEquals(first + 3, manager.nextId());
    }

    public void testInsertObjectWithFields() throws ObjectPersistenceException {
        ObjectData data = createData(Role.class, 99, new FileVersion("user", 13));
        data.set("Person", SerialOid.createPersistent(101));
        assertNotNull(data.get("Person"));
        data.set("Name", "Harry");
        assertNotNull(data.get("Name"));

        manager.insertObject(data);

        ObjectData read = manager.loadObjectData(data.getOid());
        assertEquals(data.getOid(), read.getOid());
        assertEquals(data.getClassName(), read.getClassName());

        assertEquals(data.get("Person"), read.get("Person"));
        assertEquals(data.get("Name"), read.get("Name"));
    }

    public void testInsertObjectWithEmptyOneToManyAssociations() throws ObjectPersistenceException {
        ObjectData data = createData(Team.class, 99, new FileVersion("user", 13));

        SerialOid coid = SerialOid.createPersistent(103);
        data.initCollection(coid, "Members");

        manager.insertObject(data);

        ObjectData read = manager.loadObjectData(data.getOid());
        assertEquals(data.getOid(), read.getOid());
        assertEquals(data.getClassName(), read.getClassName());

        ReferenceVector c = read.elements("Members");
        assertNull(c);
    }

    public void testInsertObjectWithOneToManyAssociations() throws ObjectPersistenceException {
        ObjectData data = createData(Team.class, 99, new FileVersion("user", 13));

        SerialOid coid = SerialOid.createPersistent(103);
        data.initCollection(coid, "Members");
        SerialOid oid[] = new SerialOid[3];
        for (int i = 0; i < oid.length; i++) {
            oid[i] = SerialOid.createPersistent(104 + i);
            data.addElement("Members", oid[i]);
        }
        manager.insertObject(data);

        ObjectData read = manager.loadObjectData(data.getOid());
        assertEquals(data.getOid(), read.getOid());
        assertEquals(data.getClassName(), read.getClassName());

        ReferenceVector c = read.elements("Members");
        for (int i = 0; i < oid.length; i++) {
            assertEquals(oid[i], c.elementAt(i));
        }
    }

    /*
     * public void xxxtestInsertValues() throws ObjectStoreException { NakedObjectSpecification type =
     * NakedObjects.getSpecificationLoader().loadSpecification(ValueObjectExample.class.getName()); SerialOid
     * oid = new SerialOid(99); ObjectData data = new ObjectData(type, oid);
     * 
     * 
     * Date date1 = new Date(); date1.add(1,2,3); data.saveValue("Date", date1);
     * 
     * FloatingPointNumber floatingPoint1 = new FloatingPointNumber(); floatingPoint1.setValue(3.145);
     * data.saveValue("Floating Point", floatingPoint1);
     * 
     * Label label1 = new Label(); label1.setValue("Labelled"); data.saveValue("Label", label1);
     * 
     * Logical logical1 = new Logical(); logical1.setValue(true); data.saveValue("Logical", logical1);
     * 
     * Money money1 = new Money(); money1.setValue(1233.45); data.saveValue("Money", money1);
     * 
     * Option option1 = new Option(new String[] {"Fred", "Sam", "joe"}, 1); data.saveValue("Option", option1);
     * 
     * Percentage percentage1 = new Percentage(); percentage1.setValue(95); data.saveValue("Percentage",
     * percentage1);
     * 
     * TextString textString1 = new TextString("Fred"); data.saveValue("Text String", textString1);
     * 
     * DateTime timestamp1 = new DateTime(); timestamp1.add(1,2,3); data.saveValue("Time Stamp", timestamp1);
     * 
     * Time time1 = new Time(); time1.add(1,30); data.saveValue("Time", time1);
     * 
     * URLString urlString1 = new URLString("http://nakedobjects.org/"); data.saveValue("Url String",
     * urlString1);
     * 
     * WholeNumber number1 = new WholeNumber(); number1.setValue(435422); data.saveValue("Whole Number",
     * number1);
     * 
     * 
     * manager.insert(data);
     * 
     * 
     * 
     * ObjectData object = manager.loadObjectData(oid);
     * 
     * Date date2 = new Date(); object.restoreValue("Date", date2); assertEquals(date1, date2);
     * 
     * FloatingPointNumber floatingPoint2 = new FloatingPointNumber(); object.restoreValue("Floating Point",
     * floatingPoint2); assertEquals(floatingPoint1, floatingPoint2);
     * 
     * Label label2 = new Label(); object.restoreValue("Label", label2); assertEquals(label1, label2);
     * 
     * Logical logical2 = new Logical(); object.restoreValue("Logical", logical2); assertEquals(logical1,
     * logical2);
     * 
     * Money money2 = new Money(); object.restoreValue("Money", money2); assertEquals(money1, money2);
     * 
     * Option option2 = new Option(new String [] {"Fred", "Sam", "joe"}); object.restoreValue("Option",
     * option2); assertEquals(option1, option2);
     * 
     * Percentage percentage2 = new Percentage(); object.restoreValue("Percentage", percentage2);
     * assertEquals(percentage1, percentage2);
     * 
     * Time time2 = new Time(); object.restoreValue("Time", time2); assertEquals(time1, time2);
     * 
     * DateTime timestamp2 = new DateTime(); object.restoreValue("Time Stamp", timestamp2);
     * assertEquals(timestamp1, timestamp2);
     * 
     * TextString textString2 = new TextString(); object.restoreValue("Text String", textString2);
     * assertEquals(textString1, textString2);
     * 
     * URLString urlString2 = new URLString(); object.restoreValue("Url String", urlString2);
     * assertEquals(urlString1, urlString2);
     * 
     * WholeNumber number2 = new WholeNumber(); object.restoreValue("Whole Number", number2);
     * assertEquals(number1, number2); }
     * 
     * public void xxxtestSaveValues() throws ObjectStoreException { NakedObjectSpecification type =
     * NakedObjects.getSpecificationLoader().loadSpecification(ValueObjectExample.class.getName()); SerialOid
     * oid = new SerialOid(99); ObjectData data = new ObjectData(type, oid);
     * 
     * manager.insert(data);
     * 
     * 
     * Date date1 = new Date(); date1.add(1,2,3); data.saveValue("Date", date1);
     * 
     * FloatingPointNumber floatingPoint1 = new FloatingPointNumber(); floatingPoint1.setValue(3.145);
     * data.saveValue("Floating Point", floatingPoint1);
     * 
     * Label label1 = new Label(); label1.setValue("Labelled"); data.saveValue("Label", label1);
     * 
     * Logical logical1 = new Logical(); logical1.setValue(true); data.saveValue("Logical", logical1);
     * 
     * Money money1 = new Money(); money1.setValue(1233.45); data.saveValue("Money", money1);
     * 
     * Option option1 = new Option(new String[] {"Fred", "Sam", "joe"}, 1); data.saveValue("Option", option1);
     * 
     * Percentage percentage1 = new Percentage(); percentage1.setValue(95); data.saveValue("Percentage",
     * percentage1);
     * 
     * TextString textString1 = new TextString("Fred"); data.saveValue("Text String", textString1);
     * 
     * DateTime timestamp1 = new DateTime(); timestamp1.add(1,2,3); data.saveValue("Time Stamp", timestamp1);
     * 
     * Time time1 = new Time(); time1.add(1,30); data.saveValue("Time", time1);
     * 
     * URLString urlString1 = new URLString("http://nakedobjects.org/"); data.saveValue("Url String",
     * urlString1);
     * 
     * WholeNumber number1 = new WholeNumber(); number1.setValue(435422); data.saveValue("Whole Number",
     * number1);
     * 
     * 
     * manager.save(data);
     * 
     * 
     * 
     * ObjectData object = manager.loadObjectData(oid);
     * 
     * Date date2 = new Date(); object.restoreValue("Date", date2); assertEquals(date1, date2);
     * 
     * FloatingPointNumber floatingPoint2 = new FloatingPointNumber(); object.restoreValue("Floating Point",
     * floatingPoint2); assertEquals(floatingPoint1, floatingPoint2);
     * 
     * Label label2 = new Label(); object.restoreValue("Label", label2); assertEquals(label1, label2);
     * 
     * Logical logical2 = new Logical(); object.restoreValue("Logical", logical2); assertEquals(logical1,
     * logical2);
     * 
     * Money money2 = new Money(); object.restoreValue("Money", money2); assertEquals(money1, money2);
     * 
     * Option option2 = new Option(new String [] {"Fred", "Sam", "joe"}); object.restoreValue("Option",
     * option2); assertEquals(option1, option2);
     * 
     * Percentage percentage2 = new Percentage(); object.restoreValue("Percentage", percentage2);
     * assertEquals(percentage1, percentage2);
     * 
     * Time time2 = new Time(); object.restoreValue("Time", time2); assertEquals(time1, time2);
     * 
     * DateTime timestamp2 = new DateTime(); object.restoreValue("Time Stamp", timestamp2);
     * assertEquals(timestamp1, timestamp2);
     * 
     * TextString textString2 = new TextString(); object.restoreValue("Text String", textString2);
     * assertEquals(textString1, textString2);
     * 
     * URLString urlString2 = new URLString(); object.restoreValue("Url String", urlString2);
     * assertEquals(urlString1, urlString2);
     * 
     * WholeNumber number2 = new WholeNumber(); object.restoreValue("Whole Number", number2);
     * assertEquals(number1, number2);
     * 
     * 
     * 
     *  }
     * 
     */

    private ObjectData createData(final Class cls, final long id, final FileVersion version) {

        NakedObjectSpecification type = NakedObjectsContext.getReflector().loadSpecification(cls.getName());
        SerialOid oid = SerialOid.createPersistent(id);
        return new ObjectData(type, oid, version);

    }

}
// Copyright (c) Naked Objects Group Ltd.
